unit Dwsutils;

interface

uses Classes;

type
{$ifndef Win32}
  ShortString = string;
{$endif}
  TCharSet = set of Char;

  TInternetHeaders = class
    FStrings : TStringList;
    function GetValue(name : string) : string;
    procedure SetValue(name : string; const value : string);
    function GetHeader(idx : integer) : string;
    function GetCount : Integer;
  public
    constructor Create;
    destructor Destroy; override;

		procedure Assign( x : TInternetHeaders);

    procedure Clear;

    procedure AddHeader( h : string);
    function HasHeader(const s : string) : Boolean;

    property Count : Integer read GetCount;
    property Header[name : string] : string
      read GetValue write SetValue; default;
    property Lines[idx : integer] : string read GetHeader;
  end;

  function InternetDateToDateTime( const d : string) : TDateTime;
  function DateTimeToInternetDate( d : TDateTime; GMTShift : Integer) : string;
  function GetTokenAnyOf( var sString: String; const s : TCharSet ): String;
  function GetToken( var sString: string; const sDelim: String ): String;
  function PosAnyOf( const sString: String; const s : TCharSet ): Integer;
  function PosFrom( const sString: String; start : Integer; d : Char ): Integer;
  function PosFromAnyOf(const sString: String; start : Integer; const s : TCharSet ): Integer;
  function ZeroPad( n : Longint; w : Integer) : string;
  function QuotedBy( const s : string; lq, rq : Char) : Boolean;
  function GetQuotableToken(
	  var sString: String; const quotes, delims : TCharSet ): String;

  function StripChar(const Strip, S: String): string;
  function StrTokenExt(const Delim, Pairs: string; Var S: string): string;
  function StrToken(const Delim: string; Var S: string): string;

  function GetFileSize(const FileName: string): LongInt;

{$ifndef Win32}
  procedure SetLength( var s : string; l : Byte);
  function Trim( const s: string) : string;
  function TrimLeft( const s: string) : string;
  function TrimRight( const s : string) : string;
{$endif}

implementation

uses SysUtils;

{$ifndef Win32}
procedure SetLength( var s : string; l : Byte);
begin
  s[0] := Char(l);
end;

function TrimLeft( const s : string) : string;
var
  I, L : Integer;
begin
  I := 1;
  L := Length(s);
  while (s[I] in [#0..#32]) and (I<=L) do Inc(I);
  Result := Copy( s, I, L);
end;

function TrimRight( const s : string) : string;
var
  I, L : Integer;
begin
  I := L;
  while (s[I] in [#0..#32]) and (I>1) do Dec(I);
  Result := Copy( s, 1, I);
end;

function Trim( const s : string) : string;
begin
  Result := TrimLeft(TrimRight(s));
end;

{$endif}

function QuotedBy( const s : string; lq, rq : Char) : Boolean;
begin
  Result := False;
  if Length(s) < 2 then Exit;
  Result := (s[1] = lq) and (s[Length(s)] = rq);
end;

(*********************************************
Strips a token from a string.
*********************************************)
function GetToken( var sString: String; const sDelim: String ): String;
var
  nPos: integer;
begin
  nPos := Pos( sDelim, sString );
  if nPos > 0 then
     begin
        Result := Copy( sString, 1, nPos-1 );
        sString := Copy( sString, nPos + Length(sDelim),
          Length( sString ) - (nPos+Length(sDelim)-1) );
     end
  else
     begin
        Result := sString;
        sString := '';
     end;
end;

function GetTokenAnyOf(
  var sString: String; const s : TCharSet ): String;
var
  nPos: integer;
begin
  nPos := PosAnyOf( sString, s );
  if nPos > 0 then
     begin
        Result := Copy( sString, 1, nPos - 1 );
        sString := Copy( sString, nPos + 1, Length( sString ) - nPos );
     end
  else
     begin
        Result := sString;
        sString := '';
     end;
end;

function ZeroPad( n : Longint; w : Integer) : string;
begin
  Result := IntToStr( n);
  while Length(Result) < w do
    Insert( '0', Result, 1);
end;

{ Legal HTTP Dates
(a)  Sun, 06 Nov 1994 08:49:37 GMT    ; RFC 822, updated by RFC 1123
(b)  Sunday, 06-Nov-94 08:49:37 GMT   ; RFC 850, obsoleted by RFC 1036
(c)  Sun Nov  6 08:49:37 1994         ; ANSI C's asctime() format
}

function InternetDateToDateTime( const d : string) : TDateTime;
var
  commapos : Integer;
  date, day, daynum, month, year, time, shift,
  hh, mm, ss : string;

  function MonthNumber( const m : string) : Integer;
  var
    I : Integer;
  begin
    Result := 1; { default to Jan }
    for I := 1 to 12 do
      if CompareText( m, ShortMonthNames[I]) = 0 then
      begin
        Result := I;
        Exit;
      end;
  end;

begin
{ Result := 0; }
  date := d;
  commapos := Pos( ',', date);

  day := '';
  daynum := '';
  month := '';
  year := '';
  time := '';
  shift := '';
  hh := '';
  ss := '';
  mm := '';

  if commapos <> 0 then
  begin
    { We have (a) or (b) type }
    day := GetToken(date, ', ');
    if Length(day) = 3 then
    begin { type (a) }
      daynum := GetToken( date, ' ');
      month := GetToken( date, ' ');
      year := GetToken( date, ' ');
      time := GetToken( date, ' ');
      shift := d;
    end else
    begin { type (b) }
      daynum := GetToken( date, '-');
      month := GetToken( date, '-');
      year := GetToken( date, ' ');
      year := IntToStr(StrToIntDef( year, 0)+1900);
      time := GetToken( date, ' ');
      shift := d;
    end;
  end else
  begin
    { We have (c) type, or invalid }
    month := GetToken( date, ' ');
    daynum := GetToken( date, ' ');
    time := GetToken( date, ' ');
    year := date;
  end;

  hh := GetToken(time, ':');
  mm := GetToken(time, ':');
  ss := GetToken(time, ':');

  Result :=
    EncodeDate( StrToIntDef(year, 0), MonthNumber(month),
      StrToIntDef(daynum, 1))+
    EncodeTime( StrToIntDef(hh, 0), StrToIntDef(mm, 0),
      StrToIntDef(ss, 0), 0);
end;

{ Convert datetime to Internet format according to prevalent RFC822 style

  Mon, 17 Mar 1997 17:06:30 -0400
}
function DateTimeToInternetDate( d : TDateTime; GMTShift : Integer) : string;
var
  shift : string;
begin
  shift := ZeroPad( Abs(GMTShift), 4);
  if GMTShift < 0 then
    Insert( '-', shift, 1)
  else
    Insert( '+', shift, 1);
  Result := FormatDateTime( 'ddd, d mmm yyyy hh:nn:ss ', d)+shift;
end;

function PosFrom( const sString: String; start : Integer; d : Char ): Integer;
var
  I, C : Integer;
begin
  C := Length(sString);
  Result := 0;
  for I := start to C do
    if sString[I] = d then
    begin
      Result := I;
      Break;
    end;
end;

function PosAnyOf(
  const sString: String; const s : TCharSet ): Integer;
begin
  Result := PosFromAnyOf( sString, 1, s);
end;

function PosFromAnyOf(
  const sString: String; start : Integer; const s : TCharSet ): Integer;
var
  I, C : Integer;
begin
  C := Length(sString);
  Result := 0;
  if C = 0 then Exit;
  for I := start to C do
    if sString[I] in s then
    begin
      Result := I;
      Break;
    end;
end;

function StripChar(const Strip, S: String): string;
var
  I: integer;
begin
  Result := S;

  { Strip from front }
  I := 1;
  while (I <= Length(Result)) and (Pos(Result[I], Strip) > 0) do
    Inc(I);
  if I > 1 then
    Delete(Result,1,I-1);

  { Strip from back }
  I := Length(Result);
  while (I > 0) and (Pos(Result[I], Strip) > 0) do
    Dec(I);
  if I < Length(Result) then
    Delete(Result, I+1, Length(Result));
end;


function StrTokenExt(const Delim, Pairs: string; Var S: string): string;
var
  I, P, D, T : integer;
begin
  D := length(S);
  for I := 1 to Length(Delim) do
  begin
    T := pos(Delim[I], S);
    if (T <> 0) and (T < D) then
      D := T;
  end;

  P := length(S);
  I := 1;
  while I < length(pairs) do
  begin
    T := pos(Pairs[I], S);
    if (T <> 0) and (T < P) then
      P := T;
    inc(I, 2);
  end;

  if (P < D) and (P <> Length(S)) then
  begin
    T := Pos(S[P], Pairs)+1;
    D := P + 1;
    while (D <> length(S)) and (S[D] <> Pairs[T]) do
      inc(D);
    Result := StripChar(Delim, Copy(S, T, D-2));
    S := StripChar(Delim, copy(S, D+1, Length(S)));
  end
  else
  begin
    Result := StripChar(Delim, Copy(S, 1, D));
    S := StripChar(Delim, copy(S, D+1, Length(S)));
  end;
end;

function StrToken(const Delim: string; Var S: string): string;
begin
  Result := StrtokenExt(Delim, '', S);
end;


{ TInternetHeaders }

constructor TInternetHeaders.Create;
begin
  inherited Create;
  FStrings := TStringList.Create;
end;

destructor TInternetHeaders.Destroy;
begin
  FStrings.Free;
  inherited Destroy;
end;

procedure TInternetHeaders.Assign( x : TInternetHeaders);
begin
	FStrings.Assign(x.FStrings);
end;

procedure TInternetHeaders.Clear;
begin
  FStrings.Clear;
end;

function TInternetHeaders.GetCount : Integer;
begin
  Result := FStrings.Count;
end;

function TInternetHeaders.GetValue(name : string) : string;
begin
  result := FStrings.Values[name];
end;

procedure TInternetHeaders.SetValue(name : string; const value : string);
begin
  FStrings.Values[name] := value;
end;

function TInternetHeaders.GetHeader(idx : integer) : string;
var
  hdr   : string;
  p     : integer;
begin
  hdr := FStrings.Strings[idx];
  p := Pos('=', hdr);
  System.Delete(hdr, p, 1);
  System.Insert(': ', hdr, p);
  Result := hdr;
end;

procedure TInternetHeaders.AddHeader( h : string);
var
  name, val : string;
begin
  name := GetToken( h, ':');
  val := TrimLeft(h);
  SetValue( name, val);
end;

function TInternetHeaders.HasHeader(const s : string) : Boolean;
var
	I, C : Integer;
begin
	C := FStrings.Count;
  Result := False;
  for I := 0 to C-1 do
		if CompareText(s, Copy(FStrings[I], 1, Length(s))) = 0 then
    begin
    	Result := True;
      Exit;
    end;
end;

function GetFileSize(const FileName: string): LongInt;
var
  SearchRec: TSearchRec;
begin
  if SysUtils.FindFirst(ExpandFileName(FileName), faAnyFile, SearchRec) = 0 then
    Result := SearchRec.Size
  else Result := -1;
  SysUtils.FindClose(SearchRec);
end;

function GetQuotableToken(
	var sString: String; const quotes, delims : TCharSet ): String;
var
	IsQuoted : Boolean;
  QuoteChar : Char;
begin
  Result := '';
	if Length(sString) = 0 then Exit;

	IsQuoted := sString[1] in quotes;

	if IsQuoted then
  begin
		Quotechar := sString[1];
    Delete( sString, 1, 1); { Remove first quote char }
  	Result := GetToken( sString, QuoteChar); {get rest of the token! }
    Delete( sString, 1, 1);  { lose the ' ' delim that was after last quote!}
  end else
  	Result := GetTokenAnyOf(sString, delims);
end;

end.
