unit Timesock;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, dWinsock, BuffSock;

type
  TTimeState = ( tsNone, tsSendingTime );

  TTimeSocket = class;

  TTimeSocketNotifyEvent = procedure (Sender : TTimeSocket) of object;

  TTimeSocket = class(TBufferedSocket)
  protected
    FBuff : Longint;
    FLastTime : TDateTime;
    FTimeState : TTimeState;
    FOnTimeWritten : TTimeSocketNotifyEvent;

    procedure DoReadBufferFull; override;
    procedure DoWriteBufferEmpty; override;
  public
    constructor Create(AParent: TSockCtrl); override;

    procedure Close; override;

    function ReadTime( var d : TDateTime; gmtshift : Integer;
      dst : Boolean) : Boolean;
    procedure WriteTime( d : TDateTime; gmtshift : Integer;
      dst : Boolean);

    property OnTimeWritten : TTimeSocketNotifyEvent
      read FOnTimeWritten write FOnTimeWritten;
    property LastTime : TDateTime read FLastTime;
  end;

  TTimeReceivedEvent =
    procedure ( Sender : TObject; ATime : TDateTime) of object;

  TTimeClient = class(TCustomClientSocket)
  private
    { Private declarations }
  protected
    { Protected declarations }
    FDST, FActive : Boolean;
    FOnTimeReceived : TTimeReceivedEvent;
    FOnTimeFailed : TNotifyEvent;
    FTime : TDateTime;
    FTimeServer, FTimePort : string;
    FGMTShift : Integer;
    procedure DoTimeout( var Socket : TSocketBase ); override;
    procedure DoConnect( var Socket : TSocketBase ); override;
    procedure DoDisconnect( var Socket : TSocketBase ); override;
    procedure DoRead( var Socket : TSocketBase ); override;

    procedure DoTimeFailed;
    procedure DoTimeReceived;

    procedure Reset;
  public
    { Public declarations }
    constructor Create( AOwner : TComponent); override;

    procedure GetTime;

    property Active : Boolean read FActive;
  published
    { Published declarations }

    property DaylightSavingsTime : Boolean read FDST write FDST;
    property GMTShift : Integer read FGMTShift write FGMTShift;
    property Timeout;
    property TimePort : string read FTimePort write FTimePort;
    property TimeServer : string read FTimeServer write FTimeServer;
    property OnTimeReceived : TTimeReceivedEvent
      read FOnTimeReceived write FOnTimeReceived;
    property OnTimeFailed : TNotifyEvent
      read FOnTimeFailed write FOnTimeFailed;

    property OnConnect;
  end;

  TTimeSentEvent = procedure ( Sender : TObject; Socket : TTimeSocket) of object;

  TTimeServer = class(TCustomBasicServerSocket)
  private
    { Private declarations }
  protected
    { Protected declarations }
    FDST, FActive : Boolean;
    FOnTimeSent : TTimeSentEvent;
    FTimePort : string;
    FGMTShift : Integer;
    procedure DoAccept( var Socket : TSocketBase ); override;

    procedure TimeSent( s : TTimeSocket);

    procedure Reset;

    function GetActive : Boolean;
    procedure SetActive(a : Boolean);
  public
    { Public declarations }
    constructor Create( AOwner : TComponent); override;

    { Toggles Listening }
    property Active : Boolean read GetActive write SetActive;

  published
    { Published declarations }

    property DaylightSavingsTime : Boolean read FDST write FDST;
    property GMTShift : Integer read FGMTShift write FGMTShift;
    property TimePort : string read FTimePort write FTimePort;
    property OnTimeSent : TTimeSentEvent
      read FOnTimeSent write FOnTimeSent;
      
    property RestrictedHosts;
    property Restriction;
    property AllowOnlyIfReverseDNS;
  end;

implementation

{ ////////////////////////////////////////////////////////////////
//  IntToDateTime
//
//  Convert a Internet TIME formatted longint to a Delphi TDateTime.
//
//  Args:
//    tl      the Internet TIME value
//    gmt     GMT shift in hours.
//    dst     daylight saving time flag
}

function IntToDateTime(tl : longint; gmt : integer; dst : boolean) : TDateTime;
var
  tf    : double;
  dt    : TDateTime;
begin
  { Workaround for the lack of unsigned long in Delphi. }
  tf := tl and MaxLongInt;
  if tl < 0 then
    begin
      tf := tf + MaxLongInt;
      tf := tf + 1;
    end;

  { Take DST setting into account. }
  if dst then
    tf := tf + 3600;

  { Take GMT setting into account. }
  tf := tf + gmt * 3600;

  { Convert to Delphi TDateTime format by dividing by }
  { number of seconds per day. }
  dt := tf / 86400;

  { Adjust for the difference in time epoch between
    Delphi (12/30/1899) and Internet TIME (1/1/1900). }
  dt := dt + (EncodeDate(1900, 1, 1));

  Result := dt;
end;

{////////////////////////////////////////////////////////////////
//  DateTimeToInt
}

function DateTimeToInt(dt : TDateTime; gmt : integer; dst : boolean) : longint;
var
  h, l  : longint;
  tl    : longint;
  tf, temp    : double;
begin
  { Adjust for the difference in time epoch between
   Delphi (12/30/1899) and Internet TIME (1/1/1900). }
  dt := dt - (EncodeDate(1900, 1, 1));

  tf := dt * 86400;

  { Take DST setting into account. }
  if dst then
    tf := tf - 3600;

  { Take GMT setting into account. }
  tf := tf - gmt * 3600;

  { Workaround for the lack of unsigned long in Delphi. }
  h := trunc(tf / 65536);
  temp := 65536;
  temp := temp * h;
  l := trunc(tf - temp);
  tl := l + (h shl 16);

  Result := tl;
end;

{ ************** TTimeSocket *************

  This class is derived from TStreamSocket and provides
  handling of the very simple Internet Time protocol
  for both clients AND servers.

  This socket is then used by the components
  in calls to TCustomClientSocket.Open and
  TCustomClientSocket.Listen

}

constructor TTimeSocket.Create(AParent: TSockCtrl);
begin
  inherited Create( AParent );

  FBuff := 0;
  ReadBufferSize := 4;  { Time is only 4 bytes so this is enough }
  WriteBufferSize := 4;
  FLastTime := 0;

  FTimeState := tsNone;
end;

function TTimeSocket.ReadTime( var d : TDateTime; gmtshift : Integer;
  dst : Boolean) : Boolean;
begin
  Result := ReadBlockFromBuffer( FBuff, 4);
  if not Result then Exit;

  d := IntToDateTime( WinSockAPI.ntohl(FBuff), gmtshift, dst);
  FLastTime := d;
end;

procedure TTimeSocket.WriteTime( d : TDateTime; gmtshift : Integer;
  dst : Boolean);
var
  ctime : Longint;
begin
  FTimeState := tsSendingTime;

  FLastTime := d;
  ctime := WinsockAPI.htonl(DateTimeToInt(d, gmtshift, dst));
  if not WriteBlockToBuffer( ctime, 4) then
    raise Exception.Create( 'TimeSocket write buffer too small!');
end;

procedure TTimeSocket.DoWriteBufferEmpty;
begin
  if FTimeState = tsSendingTime then
    if Assigned( FOnTimeWritten) then FOnTimeWritten( Self );

  FTimeState := tsNone;

  inherited DoWriteBufferEmpty;
end;

procedure TTimeSocket.DoReadBufferFull;
begin
{ This is a dummy proc. We stop TBufferedSocket calling
  the ReadBufferFull event and raising an exception because the
  buffer is still full (because we do no read the data in the
  ReadBufferFull event, we read it in ReadTime }
end;

procedure TTimeSocket.Close;
begin
  FTimeState := tsNone;
  FBuff := 0;
  FLastTime := 0;

  inherited Close;
end;

{ ************** TTimeClient ************* }

constructor TTimeClient.Create( AOwner : TComponent);
begin
  inherited Create(AOwner);

  FActive := False;
  FOnTimeReceived := nil;
  FOnTimeFailed := nil;
  FTime := 0;
  FTimeServer := '';
  FTimePort := 'time';
  FDST := False;
  FGMTShift := 0;

end;

procedure TTimeClient.GetTime;
begin
  FTime := 0;
  FActive := True;
  try
    Port := FTimePort;
    Address := FTimeServer;
    Open( TTimeSocket);
  except
    DoTimeFailed;
  end;
end;

procedure TTimeClient.Reset;
begin
  Close;
  FActive := False;
  if FTime <> 0 then DoTimeReceived;
end;

procedure TTimeClient.DoTimeout( var Socket : TSocketBase );
begin
  inherited DoTimeout(Socket);
  DoTimeFailed;
  Reset;
end;

procedure TTimeClient.DoConnect( var Socket : TSocketBase );
begin
  inherited DoConnect(Socket);
  if Socket.LastError <> 0 then DoTimeFailed;
end;

procedure TTimeClient.DoDisconnect( var Socket : TSocketBase );
begin
  inherited DoDisconnect(Socket);
  FActive := False;
end;

procedure TTimeClient.DoRead( var Socket : TSocketBase );
begin
  inherited DoRead(Socket);
  if (Socket as TTimeSocket).ReadTime( FTime, FGMTShift, FDST) then
    Reset;
end;

procedure TTimeClient.DoTimeFailed;
begin
  if Assigned( FOnTimeFailed) then FOnTimeFailed(Self);
end;

procedure TTimeClient.DoTimeReceived;
begin
  if Assigned( FOnTimeReceived) then FOnTimeReceived(Self, FTime);
end;

{ ********* TTimeServer ********** }

constructor TTimeServer.Create( AOwner : TComponent);
begin
  inherited Create(AOwner);

  FActive := False;
  FOnTimeSent := nil;
  FTimePort := 'time';
  FDST := False;
  FGMTShift := 0;
end;

function TTimeServer.GetActive : Boolean;
begin
  try
    Result := Conn.Listening
  except
    Result := False;
  end;
end;

procedure TTimeServer.SetActive( a : Boolean);
begin
  if a = FActive then Exit;

  if a then
  begin
    FActive := a;
    Port := FTimePort;
    Listen( TTimeSocket);
  end else Reset;
end;

procedure TTimeServer.Reset;
begin
  CloseDown;
  FActive := False;
end;

procedure TTimeServer.DoAccept( var Socket : TSocketBase );
begin
  inherited DoAccept(Socket);
  with (Socket as TTimeSocket) do
  begin
    OnTimeWritten := TimeSent;
    WriteTime( Now, FGMTShift, FDST);
  end;
end;

procedure TTimeServer.TimeSent( s : TTimeSocket);
begin
  if Assigned( FOnTimeSent) then FOnTimeSent(Self, s);
  s.Close;
end;

end.
